/**
 **	Database.C
 **
 **	Published / author: 2001-02-15 / grymse@telia.com
 **/

/*
Copyright (C) 2001  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql/mysql.h>
#ifdef THREAD_SAFE
#ifdef USE_SDL
#include <SDL.h>
#else
#include <pthread.h>
#endif // USE_SDL
#endif // THREAD_SAFE

#include "Database.H"

#define DEB(x) 


Database::Database(void)
:host(NULL)
,user(NULL)
,password(NULL)
,database(NULL)
,opendbbase(NULL)
,errc(0)
{
#ifdef THREAD_SAFE
#ifdef USE_SDL
	m_lock = SDL_CreateMutex();
#else
	pthread_mutex_init(&m_mutex, NULL);
#endif // USE_SDL
#endif // THREAD_SAFE
}


Database::Database(char *h,char *u,char *p,char *d)
:host(new char[strlen(h) + 1])
,user(new char[strlen(u) + 1])
,password(new char[strlen(p) + 1])
,database(new char[strlen(d) + 1])
,opendbbase(NULL)
,errc(0)
{
#ifdef THREAD_SAFE
#ifdef USE_SDL
	m_lock = SDL_CreateMutex();
#else
	pthread_mutex_init(&m_mutex, NULL);
#endif // USE_SDL
#endif // THREAD_SAFE
	strcpy(host,h);
	strcpy(user,u);
	strcpy(password,p);
	strcpy(database,d);
	
	freedb(grabdb());		// open one connection
}


Database::~Database(void)
{
	OPENDB *odb;

	if (host)
	{
		delete host;
	}
	if (user)
	{
		delete user;
	}
	if (password)
	{
		delete password;
	}
	if (database)
	{
		delete database;
	}
#ifdef THREAD_SAFE
	MutexLock();
#endif
	for (odb = opendbbase; odb; odb = odb -> next)
	{
		mysql_close(&odb -> mysql);
	}
	while (opendbbase)
	{
		odb = opendbbase;
		opendbbase = opendbbase -> next;
		if (odb -> busy)
		{
			fprintf(stderr,"destroying Database object before Connect object(s)\n");
		}
		delete odb;
	}
#ifdef THREAD_SAFE
	MutexUnlock();
#endif
#ifdef THREAD_SAFE
#ifdef USE_SDL
	SDL_DestroyMutex(m_lock);
#else
	pthread_mutex_destroy(&m_mutex);
#endif // USE_SDL
#endif // THREAD_SAFE
}

OPENDB *Database::grabdb(void)
{
	OPENDB *odb;
DEB(	int q = 0;)

#ifdef THREAD_SAFE
	MutexLock();
#endif
	for (odb = opendbbase; odb; odb = odb -> next)
	{
		if (!odb -> busy)
		{
			break;
		}
DEB(		q++;)
	}
	if (!odb)
	{
		odb = new OPENDB;
		if (!mysql_init(&odb -> mysql))
		{
			fprintf(stderr,"mysql_init() failed\n");
			errc = 1;
		}
		if (!mysql_connect(&odb -> mysql,host,user,password))
		{
			fprintf(stderr,"mysql_connect(%s,%s,***) failed\n",host,user);
			errc = 2;
		}
		if (mysql_select_db(&odb -> mysql,database))
		{
			fprintf(stderr,"mysql_select_db(%s,%s,%s,%s) failed\n",host,user,password,database);
			errc = 3;
		}
		odb -> busy = 1;
		odb -> next = opendbbase;
		opendbbase = odb;
DEB(		q++;
		fprintf(stderr,"Database::grabdb():  %d handle%s in use\n",q,(q == 1) ? "" : "s");)
	}
	else
	{
		odb -> busy++;
	}
#ifdef THREAD_SAFE
	MutexUnlock();
#endif
	return odb;
}


void Database::freedb(OPENDB *odb)
{
	odb -> busy = 0;
}


short Database::errcode(void)
{
	return errc;
}


void Database::debug(short val)
{
	_debug = val;
}


short Database::debug(void)
{
	return _debug;
}
