/**
 **	Query.h
 **
 **	Published / author: 2001-01-04 / grymse@telia.com
 **/

/*
Copyright (C) 2001  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _QUERY_H
#define _QUERY_H


#include <syslog.h>
#include <Database.h>
#include <Ticks.h>


class Query {
public:
	Query::Query(void) {
		db = NULL;
		odb = NULL;
		res = NULL;
		row = NULL;
		_debug = _syslog = 0;
	}
	Query::Query(Database *dbin) {
		db = dbin;
		odb = db -> grabdb();
		res = NULL;
		row = NULL;
		_debug = _syslog = 0;

		_debug = db -> debug();
	}
	short Query::execute(char *sql) {		// query, no result
		if (db && odb && res)
			fprintf(stderr,"query busy\n");
		if (db && odb && !res)
		{
			Ticks clock;
			if (db -> qlog())
			{
				fprintf(db -> logfile(),"%s\n",sql);
				fflush(db -> logfile());
			}
			if (mysql_query(&odb -> mysql,sql))
				fprintf(stderr,"query failed: '%s'\n",sql);
			else
				return 1;
			if (db -> qlog())
			{
				clock.Stop();
				clock.Time(sql);
				fprintf(db -> logfile(),"Time: %s\n\n",sql);
				fflush(db -> logfile());
			}
		}
		return 0;
	}
	Query::Query(Database *dbin,char *sql) {
		db = dbin;
		odb = db -> grabdb();
		res = NULL;
		row = NULL;
		_debug = _syslog = 0;

		_debug = db -> debug();
		if (!execute(sql))
			fprintf(stderr,"query failed: '%s'\n",sql);
	}

// destructor

	Query::~Query(void) {
//printf("~Query()\n");
		if (res)
			mysql_free_result(res);
		if (odb)
			db -> freedb(odb);
	}

// methods using db specific api calls

	MYSQL_RES *Query::get_result(char *sql) {	// query, result
		if (db && odb && res)
			fprintf(stderr,"query busy\n");
		if (db && odb && !res)
		{
			if (Query::execute(sql))
				res = mysql_store_result(&odb -> mysql);
			else
			{
				fprintf(stderr,"query failed: '%s'\n",sql);
				if (_debug)
					printf("\nquery failed: '%s'<br>\n",sql);
			}
		}
		return res;
	}
	void Query::free_result(void) {
		if (db && odb && res)
		{
			mysql_free_result(res);
			res = NULL;
			row = NULL;
		}
	}
	MYSQL_ROW Query::fetch_row(void) {
		rowcount = 0;
		return db && odb && res ? row = mysql_fetch_row(res) : NULL;
	}
	long Query::insert_id(void) {
		if (db && odb)
			return mysql_insert_id(&odb -> mysql);
		else
			return -1;
	}
	long Query::num_rows(void) {
		return db && odb && res ? mysql_num_rows(res) : 0;
	}

// data retreival methods

	char *Query::getstr(int x) {
		if (db && odb && res && row)
			return row[x] ? row[x] : (char *)"";
		else
			return NULL;
	}
	char *Query::getstr(void) {
		return Query::getstr(rowcount++);
	}
	double Query::getnum(int x) {
		return db && odb && res && row && row[x] ? atof(row[x]) : 0;
	}
	long Query::getval(int x) {
		return db && odb && res && row && row[x] ? atol(row[x]) : 0;
	}
	double Query::getnum(void) {
		return getnum(rowcount++);
	}
	long Query::getval(void) {
		return getval(rowcount++);
	}
	double Query::get_num(char *sql) {
		double l = 0;
		if (Query::get_result(sql))
		{
			if (Query::fetch_row())
				l = Query::getnum();
			Query::free_result();
		}
		return l;
	}
	long Query::get_count(char *sql) {
		long l = 0;
		if (Query::get_result(sql))
		{
			if (Query::fetch_row())
				l = Query::getval();
			Query::free_result();
		}
		return l;
	}
	char *Query::get_string(char *sql) {
	static 	char slask[32000];
		*slask = 0;
		if (Query::get_result(sql))
		{
			if (Query::fetch_row())
				strcpy(slask,Query::getstr());
			Query::free_result();
		}
		return slask;
	}
	void Query::debug(short val) { _debug = val; }
	void Query::syslog(short val) { _syslog = val; }

//MYSQL_FIELD *	STDCALL mysql_fetch_field(MYSQL_RES *result);

	MYSQL_FIELD *Query::fetch_field(void) {
		return db && odb && res ? mysql_fetch_field(res) : NULL;
  	}
	char *Query::fetch_fieldname(void) {
		MYSQL_FIELD *field = db && odb && res ? mysql_fetch_field(res) : NULL;
		return field ? field -> name : (char *)"";
  	}

	void Query::safestr(char *s) {
		char *s2 = new char[strlen(s) * 2 + 2];
		int i,j = 0;
		*s2 = 0;
		for (i = 0; i < (int)strlen(s); i++)
			switch (s[i])
			{
			case '\'':
			case '\\':
			case 34:
				s2[j++] = '\\';
			default:
				s2[j++] = s[i];
			}
		s2[j] = 0;
		strcpy(s,s2);
	}
	void Query::unsafestr(char *s) {
		int i;
		for (i = 0; i < (int)strlen(s); i++)
			if (s[i] == '\\')
				memmove(s + i,s + i + 1,strlen(s + i) + 1);
	}
/*
& &amp; 
< &lt; 
> &gt; 
" &quot; 
' &apos; 
*/
	char *Query::xmlsafestr(char *s) {
	static	char str[999];
		int i;
		*str = 0;
		for (i = 0; i < (int)strlen(s); i++)
		{
			switch (s[i])
			{
			case '&':
				strcat(str,"&amp;");
				break;
			case '<':
				strcat(str,"&lt;");
				break;
			case '>':
				strcat(str,"&gt;");
				break;
			case '"':
				strcat(str,"&quot;");
				break;
			case '\'':
				strcat(str,"&apos;");
				break;
			default:
				sprintf(str + strlen(str),"%c",s[i]);
			}
		}
		return str;
	}

private:
	Database *db;
	OPENDB *odb;
#ifdef _mysql_h
	MYSQL_RES *res;
	MYSQL_ROW row;
#endif
	short rowcount;
	short _debug;
	short _syslog;
};


#endif // _QUERY_H
